# `String.repeat()` – 在 Java 中重复字符串 N 次

> 原文： [https://howtodoinjava.com/java11/repeat-string-n-times/](https://howtodoinjava.com/java11/repeat-string-n-times/)

通过简单的 Java 程序，学习将给定的字符串重复 N 次，以产生包含所有重复的新字符串。 我们将使用方法`Sting.repeat(N)`（自 [Java 11](https://howtodoinjava.com/java11/features-enhancements/) 起），并使用可用于 [Java 10](https://howtodoinjava.com/java10/java10-features/) 的正则表达式。

## 1\. `String.repeat()` API（自 Java 11 起）

此方法返回一个字符串，该字符串的值是重复`count`次的给定字符串的连接。 如果字符串为空或`count`为零，则返回空字符串。

#### 1.1 语法

```java
/**
* Parameters:
* count - number of times to repeat
* 
* Returns:
* A string composed of this string repeated count times or the empty string if this string is empty or count is zero
* 
* Throws:
* IllegalArgumentException - if the count is negative.
*/

public String repeat​(int count)

```

#### 1.2 示例

Java 程序将字符串`"Abc"`重复 3 次。

```java
public class Main 
{
	public static void main(String[] args) 
	{
		String str = "Abc";

		System.out.println( str.repeat(3) );
	}
}

```

程序输出。

```java
AbcAbcAbc

```

## 2\. 使用正则表达式重复字符串（直到 Java 10）

如果您正在使用 JDK < = 10，则可以考虑使用正则表达式将字符串重复 N 次。

Java program to repeat string ‘Abc’ to 3 times.

```java
public class Main 
{
	public static void main(String[] args) 
	{
		String str = "Abc";

		String repeated = new String(new char[3]).replace("\0", str);

		System.out.println(repeated);
	}
}

```

程序输出：

```java
AbcAbcAbc

```

## 3\. Apache Common 的`StringUtils`类

如果不是正则表达式，则可以使用`StringUtils`类及其方法[`repeat(times)`](https://commons.apache.org/proper/commons-lang/javadocs/api-3.1/org/apache/commons/lang3/StringUtils.html#repeat(java.lang.String,%20int))。

```java
import org.apache.commons.lang3.StringUtils;

public class Main 
{
	public static void main(String[] args) 
	{
		String str = "Abc";

		String repeated = StringUtils.repeat(str, 3);

		System.out.println(repeated);
	}
}

```

程序输出：

```java
AbcAbcAbc

```

向我提供有关**如何在 Java** 中将字符串重复 N 次的问题。

学习愉快！