# `String.strip()` – 删除开头和结尾的空格

> 原文： [https://howtodoinjava.com/java11/strip-remove-white-spaces/](https://howtodoinjava.com/java11/strip-remove-white-spaces/)

在 [Java 11](https://howtodoinjava.com/java11/features-enhancements/) 中，学习如何使用`String`类的`strip()`，`stripLeading()`和`stripTrailing()`方法从给定的字符串中删除不需要的空格。

## 1\. `String.strip()`API

从 Java 11 开始，`String`类包含 3 个以上的方法，这些方法有助于消除多余的空白。 这些方法使用[`Character.isWhitespace(char)`](https://docs.oracle.com/en/java/javase/12/docs/api/java.base/java/lang/Character.html#isWhitespace(int)) 方法确定空白字符。

*   **`String.strip()`** – 返回其值为字符串的字符串，其中**删除了所有前导和尾随空白**。 请注意，`String.trim()`方法也会产生相同的结果。
*   **`String.stripLeading()`** – 返回其值为字符串的字符串，其中**删除了所有前导空白**。
*   **`String.stripTrailing()`** – 返回其值为字符串的字符串，其中**删除了所有尾随空白**。

```java
public class Main 
{
	public static void main(String[] args) 
	{
		String str = "  Hello World !!   ";

		System.out.println( str.strip() );			//"Hello World !!"

		System.out.println( str.stripLeading() );	//"Hello World !!   "

		System.out.println( str.stripTrailing() );	//"  Hello World !!"
	}
}

```

## 2\. 使用正则表达式修剪空白（包括制表符）

在不使用 Java 11 的情况下，可以使用正则表达式来修剪字符串周围的空白。

| 正则表达式 | 描述 |
| --- | --- |
| <code>^[\t]+&#124;[\t]+$</code> | 删除开头和结尾的空格 |
| **`^[\t]+`** | 仅删除开头的空格 |
| **`[\t]+$`** | 仅删除结尾的空格 |

```java
public class Main 
{
	public static void main(String[] args) 
	{
		String str = "  Hello World !!   ";

		System.out.println( str.replaceAll("^[ \t]+|[ \t]+$", "") );	//"Hello World !!"

		System.out.println( str.replaceAll("^[ \t]+", "") );			//"Hello World !!   "

		System.out.println( str.replaceAll("[ \t]+$", "") );			//"  Hello World !!"
	}
}

```

向我提供有关**如何在 Java 中从字符串**中删除空格和制表符的问题。

学习愉快！

参考：

[`regular-expressions.info`](https://www.regular-expressions.info/examples.html)

[Java `String.strip()` API 文档](https://docs.oracle.com/en/java/javase/12/docs/api/java.base/java/lang/String.html#strip())