# 文件`writeString()` API – 用 Java 将字符串写入文件

> 原文： [https://howtodoinjava.com/java11/write-string-to-file/](https://howtodoinjava.com/java11/write-string-to-file/)

学习使用[`Files.writeString(path, string, options)`](https://docs.oracle.com/en/java/javase/11/docs/api/java.base/java/nio/file/Files.html#writeString(java.nio.file.Path,java.lang.CharSequence,java.nio.charset.Charset,java.nio.file.OpenOption...))方法将字符串写入 Java 文件中。 该 API 已在 [Java 11](https://howtodoinjava.com/java11/features-enhancements/) 中引入。

## 1\. 文件`writeString()`方法

`java.nio.file.Files`类具有两个重载的[静态](https://howtodoinjava.com/java/basics/java-static-keyword/)方法，用于将内容写入文件。

```java
public static Path writeString​(Path path, CharSequence csq, 
							OpenOption... options) throws IOException

public static Path writeString​(Path path, CharSequence csq, 
							Charset cs, OpenOption... options) throws IOException

```

*   第一种方法使用 **UTF-8** 字符集将所有内容写入文件。
*   第一种方法等效于`writeString(path, string, StandardCharsets.UTF_8, options)`。
*   第二种方法与仅使用指定字符集的方法相同。
*   `options`指定如何打开文件。

## 2\. 文件`writeString()`示例

使用`Files.writeString()`方法将`String`写入文件的 Java 程序。

```java
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.Files;
import java.io.IOException;
import java.nio.file.StandardOpenOption;

public class Main 
{
	public static void main(String[] args) 
	{
		Path filePath = Paths.get("C:/", "temp", "test.txt");

		try 
		{
			//Write content to file
			Files.writeString(filePath, "Hello World !!", StandardOpenOption.APPEND);

			//Verify file content
			String content = Files.readString(filePath);

			System.out.println(content);
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
}

```

程序输出。

```java
Hello World !!

```

文件`c:/temp/test.txt`最初为空。

将我的问题放在评论部分。

学习愉快！