# JUnit5 `@BeforeEach`注解示例

> 原文： [https://howtodoinjava.com/junit5/before-each-annotation-example/](https://howtodoinjava.com/junit5/before-each-annotation-example/)

JUnit5 [`@BeforeEach`](http://junit.org/junit5/docs/current/api/org/junit/jupiter/api/BeforeEach.html)注解替换了 JUnit4 中的`@Before`注解。它用于表示应在当前类中的每个`@Test`方法之前执行**注解方法**。

## `@BeforeEach`注解用法

使用`@BeforeEach`注解方法，如下所示：

```java
@BeforeEach
public void initEach(){
	System.out.println("Before Each initEach() method called");
}

```

`@BeforeEach`**注解的方法不得为静态方法**，否则它将引发运行时错误。

```java
org.junit.platform.commons.JUnitException: @BeforeEach method 'public static void com.howtodoinjava.junit5.examples.JUnit5AnnotationsExample.initEach()' must not be static.
	at org.junit.jupiter.engine.descriptor.LifecycleMethodUtils.assertNonStatic(LifecycleMethodUtils.java:73)
	at org.junit.jupiter.engine.descriptor.LifecycleMethodUtils.lambda$findBeforeEachMethods$2(LifecycleMethodUtils.java:54)
	at java.util.ArrayList.forEach(ArrayList.java:1249)
	at java.util.Collections$UnmodifiableCollection.forEach(Collections.java:1080)
	at org.junit.jupiter.engine.descriptor.LifecycleMethodUtils.findBeforeEachMethods(LifecycleMethodUtils.java:54)

```

## `@BeforeEach`注解示例

让我们举个例子。 我使用了一个`Calculator`类并添加了一个`add`方法。 我将使用`@RepeatedTest`注解对其进行 5 次测试。 此注解将导致`add`测试运行 5 次。 对于每次运行的测试方法，`@BeforeEach`带注解的方法也应每次运行。

```java
package com.howtodoinjava.junit5.examples;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.RepeatedTest;
import org.junit.jupiter.api.RepetitionInfo;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInfo;
import org.junit.platform.runner.JUnitPlatform;
import org.junit.runner.RunWith;

@RunWith(JUnitPlatform.class)
public class BeforeEachTest {

	@DisplayName("Add operation test")
	@RepeatedTest(5)
	void addNumber(TestInfo testInfo, RepetitionInfo repetitionInfo) {
		System.out.println("Running test -> " + repetitionInfo.getCurrentRepetition());
		Assertions.assertEquals(2, Calculator.add(1, 1), "1 + 1 should equal 2");
	}

	@BeforeAll
	public static void init(){
		System.out.println("BeforeAll init() method called");
	}

	@BeforeEach
	public void initEach(){
		System.out.println("BeforeEach initEach() method called");
	}
}

```

其中`Calculator`类是：

```java
package com.howtodoinjava.junit5.examples;

public class Calculator 
{
	public int add(int a, int b) {
		return a + b;
	}
}

```

现在执行测试，您将看到以下控制台输出：

```java
BeforeAll init() method called
BeforeEach initEach() method called

BeforeEach initEach() method called
Running test -> 1

BeforeEach initEach() method called
Running test -> 2

BeforeEach initEach() method called
Running test -> 3

BeforeEach initEach() method called
Running test -> 4

BeforeEach initEach() method called
Running test -> 5

```

显然，每次测试方法调用时都调用`@BeforeEach`注解的`initEach()`方法。

学习愉快！

[源码下载](https://github.com/lokeshgupta1981/Junit5Examples/tree/master/JUnit5Examples)